package stockmanage

import (
	"errors"
	"github.com/gin-gonic/gin"
	model "go-admin/app/admin/models"
	orm "go-admin/common/global"
	"go-admin/logger"
	"go-admin/tools/app"
	"net/http"
)

func ErpStockList(c *gin.Context) {
	req := &model.ErpStockListReq{}
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error(err)
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "参数错误")
		return
	}

	resp, err := req.List()
	if err != nil {
		logger.Error("erp commodity list err:", err)
		app.Error(c, http.StatusInternalServerError, err, "获取失败")
		return
	}

	app.OK(c, resp, "")
	return
}

func ErpStockDetail(c *gin.Context) {
	req := &struct {
		ErpStockId     uint32 `json:"erp_stock_id"`
		ErpCommodityId uint32 `json:"erp_commodity_id"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error(err)
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "参数错误")
		return
	}
	var erpStock model.ErpStock
	err := orm.Eloquent.Table("erp_stock").Where("id=?", req.ErpStockId).Find(&erpStock).Error
	if err != nil {
		logger.Error("erp stock err:", err)
		app.Error(c, http.StatusInternalServerError, err, "获取失败")
		return
	}

	var commodities []model.ErpStockCommodity
	err = orm.Eloquent.Table("erp_stock_commodity").Where("erp_commodity_id=?", erpStock.ErpCommodityId).Find(&commodities).Error
	if err != nil {
		logger.Error("erp stock err:", err)
		app.Error(c, http.StatusInternalServerError, err, "获取失败")
		return
	}
	model.ErpStockCommodityListSetAge(commodities)
	erpStock.Commodities = commodities

	app.OK(c, erpStock, "")
	return
}

func ErpStockCommodityList(c *gin.Context) {
	req := &model.ErpStockCommodityListReq{}
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error(err)
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "参数错误")
		return
	}

	resp, err := req.List()
	if err != nil {
		logger.Error("erp commodity list err:", err)
		app.Error(c, http.StatusInternalServerError, err, "获取失败")
		return
	}

	app.OK(c, resp, "")
	return
}
